@@ -34,17 +34,37 @@ struct scaled_scene_buffer {
 	/* cached wlr_buffers for each scale */
 	struct wl_list cache;  /* struct scaled_buffer_cache_entry.link */
 	struct wl_listener destroy;
-	struct wl_listener output_enter;
-	struct wl_listener output_leave;
+	struct wl_listener outputs_update;
 	const struct scaled_scene_buffer_impl *impl;
-	/*
-	 * Pointer to the per-implementation list of scaled-scene-buffers.
-	 * This is used to share the backing wlr_buffers.
-	 */
-	struct wl_list *cached_buffers;
-	struct wl_list link; /* struct scaled_scene_buffer.cached_buffers */
+	struct wl_list link; /* all_scaled_buffers */
 };
 
+/*
+ *                                  |                 |
+ *                        .------------------.  .------------.
+ *       scaled_buffer    | new_output_scale |  | set_buffer |
+ *       architecture     ´------------------`  ´------------`
+ *                                  |                ^
+ *    .-----------------------------|----------------|-----------.
+ *    |                             v                |           |
+ *    |  .---------------.    .-------------------------.        |
+ *    |  | scaled_buffer |----| wlr_buffer LRU cache(2) |<---,   |
+ *    |  ´---------------`    ´-------------------------`    |   |
+ *    |           |                       |                  |   |
+ *    |        .------.       .--------------------------.   |   |
+ *    |        | impl |       | wlr_buffer LRU cache of  |   |   |
+ *    |        ´------`       |   other scaled_buffers   |   |   |
+ *    |                       |   with impl->equal()     |   |   |
+ *    |                       ´--------------------------`   |   |
+ *    |                          /              |            |   |
+ *    |                   not found           found          |   |
+ *    |     .-----------------------.     .-----------.      |   |
+ *    |     | impl->create_buffer() |--->| wlr_buffer |------`   |
+ *    |     ´-----------------------`    ´------------`          |
+ *    |                                                          |
+ *    ´----------------------------------------------------------`
+ */
+
 /**
  * Create an auto scaling buffer that creates a wlr_scene_buffer
  * and subscribes to its output_enter and output_leave signals.
@@ -64,8 +84,8 @@ struct scaled_scene_buffer {
  * allocations.
  *
  * Besides caching buffers for each scale per scaled_scene_buffer, we also
- * store all the scaled_scene_buffers in a per-implementer list passed as
- * @cached_buffers in order to reuse backing buffers for visually duplicated
+ * store all the scaled_scene_buffers from all the implementers in a list
+ * in order to reuse backing buffers for visually duplicated
  * scaled_scene_buffers found via impl->equal().
  *
  * All requested lab_data_buffers via impl->create_buffer() will be locked
@@ -94,10 +114,27 @@ struct scaled_scene_buffer {
 struct scaled_scene_buffer *scaled_scene_buffer_create(
 	struct wlr_scene_tree *parent,
 	const struct scaled_scene_buffer_impl *implementation,
-	struct wl_list *cached_buffers, bool drop_buffer);
+	bool drop_buffer);
 
-/* Clear the cache of existing buffers, useful in case the content changes */
-void scaled_scene_buffer_invalidate_cache(struct scaled_scene_buffer *self);
+/**
+ * scaled_scene_buffer_request_update - mark the buffer that needs to be
+ * updated
+ * @width: the width of the buffer to be rendered, in scene coordinates
+ * @height: the height of the buffer to be rendered, in scene coordinates
+ *
+ * This function should be called when the states bound to the buffer are
+ * updated and ready for rendering.
+ */
+void scaled_scene_buffer_request_update(struct scaled_scene_buffer *self,
+	int width, int height);
+
+/**
+ * scaled_scene_buffer_invalidate_sharing - clear the list of entire cached
+ * scaled_scene_buffers used to share visually dupliated buffers. This should
+ * be called on Reconfigure to force updates of newly created
+ * scaled_scene_buffers rather than reusing ones created before Reconfigure.
+ */
+void scaled_scene_buffer_invalidate_sharing(void);
 
 /* Private */
 struct scaled_scene_buffer_cache_entry {
