@@ -9,6 +9,7 @@
 #include "common/border.h"
 #include "common/buf.h"
 #include "common/font.h"
+#include "common/three-state.h"
 #include "config/touch.h"
 #include "config/tablet.h"
 #include "config/tablet-tool.h"
@@ -68,6 +69,7 @@ struct rcxml {
 	int gap;
 	enum adaptive_sync_mode adaptive_sync;
 	enum tearing_mode allow_tearing;
+	bool auto_enable_outputs;
 	bool reuse_output_mode;
 	enum view_placement_policy placement_policy;
 	bool xwayland_persistence;
@@ -82,6 +84,7 @@ struct rcxml {
 	/* theme */
 	char *theme_name;
 	char *icon_theme_name;
+	char *fallback_app_icon_name;
 	struct wl_list title_buttons_left;
 	struct wl_list title_buttons_right;
 	int corner_radius;
@@ -104,7 +107,7 @@ struct rcxml {
 	/* keyboard */
 	int repeat_rate;
 	int repeat_delay;
-	bool kb_numlock_enable;
+	enum three_state kb_numlock_enable;
 	bool kb_layout_per_window;
 	struct wl_list keybinds;   /* struct keybind.link */
 
@@ -148,6 +151,8 @@ struct rcxml {
 
 	enum resize_indicator_mode resize_indicator;
 	bool resize_draw_contents;
+	int resize_corner_range;
+	int resize_minimum_area;
 
 	struct {
 		int popuptime;
@@ -172,6 +177,7 @@ struct rcxml {
 
 	/* Menu */
 	unsigned int menu_ignore_button_release_period;
+	bool menu_show_icons;
 
 	/* Magnifier */
 	int mag_width;
