@@ -21,6 +21,10 @@ struct input_method_relay {
 	struct wl_list text_inputs; /* struct text_input.link */
 	struct wlr_input_method_v2 *input_method;
 	struct wlr_surface *focused_surface;
+
+	struct lab_set forwarded_pressed_keys;
+	struct wlr_keyboard_modifiers forwarded_modifiers;
+
 	/*
 	 * Text-input which is enabled by the client and communicating with
 	 * input-method.
