@@ -20,6 +20,8 @@ void keyboard_setup_handlers(struct keyboard *keyboard);
 void keyboard_set_numlock(struct wlr_keyboard *keyboard);
 void keyboard_update_layout(struct seat *seat, xkb_layout_index_t layout);
 void keyboard_cancel_keybind_repeat(struct keyboard *keyboard);
-bool keyboard_any_modifiers_pressed(struct wlr_keyboard *keyboard);
+void keyboard_cancel_all_keybind_repeats(struct seat *seat);
+
+uint32_t keyboard_get_all_modifiers(struct seat *seat);
 
 #endif /* LABWC_KEYBOARD_H */
