@@ -14,7 +14,8 @@ struct drawing_tablet {
 	struct seat *seat;
 	struct wlr_tablet *tablet;
 	struct wlr_tablet_v2_tablet *tablet_v2;
-	double x, y;
+	enum motion motion_mode;
+	double x, y, dx, dy;
 	double distance;
 	double pressure;
 	double tilt_x, tilt_y;
@@ -22,15 +23,13 @@ struct drawing_tablet {
 	double slider;
 	double wheel_delta;
 	struct {
-		struct wl_listener proximity;
-		struct wl_listener axis;
-		struct wl_listener tip;
-		struct wl_listener button;
 		struct wl_listener destroy;
 	} handlers;
 	struct wl_list link; /* seat.tablets */
 };
 
-void tablet_init(struct seat *seat, struct wlr_input_device *wlr_input_device);
+void tablet_init(struct seat *seat);
+void tablet_finish(struct seat *seat);
+void tablet_create(struct seat *seat, struct wlr_input_device *wlr_input_device);
 
 #endif /* LABWC_TABLET_H */
