@@ -79,37 +79,31 @@ font_width(struct font *font, const char *string)
 	return rectangle.width;
 }
 
+void
+font_get_buffer_size(int max_width, const char *text, struct font *font,
+	int *width, int *height)
+{
+	PangoRectangle text_extents = font_extents(font, text);
+	if (max_width > 0 && text_extents.width > max_width) {
+		text_extents.width = max_width;
+	}
+	*width = text_extents.width;
+	*height = text_extents.height;
+}
+
 void
 font_buffer_create(struct lab_data_buffer **buffer, int max_width,
 	const char *text, struct font *font, const float *color,
-	const float *bg_color, const char *arrow, double scale)
+	const float *bg_color, double scale)
 {
-	/* Allow a minimum of one pixel each for text and arrow */
-	if (max_width < 2) {
-		max_width = 2;
-	}
-
 	if (string_null_or_empty(text)) {
 		return;
 	}
 
-	PangoRectangle text_extents = font_extents(font, text);
-	PangoRectangle arrow_extents = font_extents(font, arrow);
+	int width, height;
+	font_get_buffer_size(max_width, text, font, &width, &height);
 
-	if (arrow) {
-		if (arrow_extents.width >= max_width - 1) {
-			/* It would be weird to get here, but just in case */
-			arrow_extents.width = max_width - 1;
-			text_extents.width = 1;
-		} else {
-			text_extents.width = max_width - arrow_extents.width;
-		}
-	} else if (text_extents.width > max_width) {
-		text_extents.width = max_width;
-	}
-
-	*buffer = buffer_create_cairo(text_extents.width + arrow_extents.width,
-			text_extents.height, scale);
+	*buffer = buffer_create_cairo(width, height, scale);
 	if (!*buffer) {
 		wlr_log(WLR_ERROR, "Failed to create font buffer");
 		return;
@@ -142,7 +136,7 @@ font_buffer_create(struct lab_data_buffer **buffer, int max_width,
 
 	PangoLayout *layout = pango_cairo_create_layout(cairo);
 	pango_context_set_round_glyph_positions(pango_layout_get_context(layout), false);
-	pango_layout_set_width(layout, text_extents.width * PANGO_SCALE);
+	pango_layout_set_width(layout, width * PANGO_SCALE);
 	pango_layout_set_text(layout, text, -1);
 	pango_layout_set_ellipsize(layout, PANGO_ELLIPSIZE_END);
 
@@ -161,13 +155,6 @@ font_buffer_create(struct lab_data_buffer **buffer, int max_width,
 	pango_cairo_update_layout(cairo, layout);
 	pango_cairo_show_layout(cairo, layout);
 
-	if (arrow) {
-		cairo_move_to(cairo, text_extents.width, 0);
-		pango_layout_set_width(layout, arrow_extents.width * PANGO_SCALE);
-		pango_layout_set_text(layout, arrow, -1);
-		pango_cairo_show_layout(cairo, layout);
-	}
-
 	g_object_unref(layout);
 
 	cairo_surface_flush(surf);
