@@ -21,11 +21,11 @@ find_default_config(void)
 struct touch_config_entry *
 touch_find_config_for_device(char *device_name)
 {
-	wlr_log(WLR_INFO, "find touch configuration for %s\n", device_name);
+	wlr_log(WLR_INFO, "find touch configuration for %s", device_name);
 	struct touch_config_entry *entry;
 	wl_list_for_each(entry, &rc.touch_configs, link) {
 		if (entry->device_name && !strcasecmp(entry->device_name, device_name)) {
-			wlr_log(WLR_INFO, "found touch configuration for %s\n", device_name);
+			wlr_log(WLR_INFO, "found touch configuration for %s", device_name);
 			return entry;
 		}
 	}
