@@ -3,6 +3,7 @@
 #include <wlr/types/wlr_scene.h>
 #include "common/graphic-helpers.h"
 #include "common/scene-helpers.h"
+#include "common/string-helpers.h"
 #include "debug.h"
 #include "input/ime.h"
 #include "labwc.h"
@@ -69,14 +70,14 @@ get_view_part(struct view *view, struct wlr_scene_node *node)
 	}
 	if (node == &view->scene_tree->node) {
 		const char *app_id = view_get_string_prop(view, "app_id");
-		if (!app_id) {
+		if (string_null_or_empty(app_id)) {
 			return "view";
 		}
 		snprintf(view_name, sizeof(view_name), "view (%s)", app_id);
 		return view_name;
 	}
-	if (node == view->scene_node) {
-		return "view->scene_node";
+	if (node == &view->content_tree->node) {
+		return "view->content_tree";
 	}
 	if (view->resize_indicator.tree
 			&& node == &view->resize_indicator.tree->node) {
