@@ -108,37 +108,6 @@ desktop_focus_view_or_surface(struct seat *seat, struct view *view,
 	}
 }
 
-struct view *
-desktop_cycle_view(struct server *server, struct view *start_view,
-		enum lab_cycle_dir dir)
-{
-	/* Make sure to have all nodes in their actual ordering */
-	osd_preview_restore(server);
-
-	struct view *(*iter)(struct wl_list *head, struct view *view,
-		enum lab_view_criteria criteria);
-	bool forwards = dir == LAB_CYCLE_DIR_FORWARD;
-	iter = forwards ? view_next_no_head_stop : view_prev_no_head_stop;
-
-	enum lab_view_criteria criteria = rc.window_switcher.criteria;
-
-	/*
-	 * Views are listed in stacking order, topmost first.  Usually the
-	 * topmost view is already focused, so when iterating in the forward
-	 * direction we pre-select the view second from the top:
-	 *
-	 *   View #1 (on top, currently focused)
-	 *   View #2 (pre-selected)
-	 *   View #3
-	 *   ...
-	 */
-	if (!start_view && forwards) {
-		start_view = iter(&server->views, NULL, criteria);
-	}
-
-	return iter(&server->views, start_view, criteria);
-}
-
 struct view *
 desktop_topmost_focusable_view(struct server *server)
 {
@@ -215,7 +184,7 @@ desktop_focus_output(struct output *output)
 }
 
 void
-desktop_update_top_layer_visiblity(struct server *server)
+desktop_update_top_layer_visibility(struct server *server)
 {
 	struct view *view;
 	struct output *output;
@@ -309,7 +278,8 @@ get_cursor_context(struct server *server)
 			case LAB_NODE_DESC_VIEW:
 			case LAB_NODE_DESC_XDG_POPUP:
 				ret.view = desc->data;
-				ret.type = ssd_get_part_type(ret.view->ssd, ret.node);
+				ret.type = ssd_get_part_type(
+					ret.view->ssd, ret.node, cursor);
 				if (ret.type == LAB_SSD_CLIENT) {
 					ret.surface = lab_wlr_surface_from_node(ret.node);
 				}
@@ -348,6 +318,7 @@ get_cursor_context(struct server *server)
 				return ret;
 			case LAB_NODE_DESC_NODE:
 			case LAB_NODE_DESC_TREE:
+			case LAB_NODE_DESC_SCALED_SCENE_BUFFER:
 				break;
 			}
 		}
