@@ -136,6 +136,9 @@ try_to_focus_next_layer_or_toplevel(struct server *server)
 {
 	struct seat *seat = &server->seat;
 	struct output *output = output_nearest_to_cursor(server);
+	if (!output) {
+		goto no_output;
+	}
 
 	enum zwlr_layer_shell_v1_layer overlay = ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY;
 	enum zwlr_layer_shell_v1_layer top = ZWLR_LAYER_SHELL_V1_LAYER_TOP;
@@ -169,6 +172,7 @@ try_to_focus_next_layer_or_toplevel(struct server *server)
 	 * Unfocus the current layer-surface and focus the topmost toplevel if
 	 * one exists on the current workspace.
 	 */
+no_output:
 	if (seat->focused_layer) {
 		seat_set_focus_layer(seat, NULL);
 	}
@@ -363,6 +367,8 @@ popup_handle_destroy(struct wl_listener *listener, void *data)
 		wl_list_remove(&popup->commit.link);
 	}
 
+	cursor_update_focus(popup->server);
+
 	free(popup);
 }
 
@@ -394,9 +400,11 @@ popup_handle_reposition(struct wl_listener *listener, void *data)
 static void popup_handle_new_popup(struct wl_listener *listener, void *data);
 
 static struct lab_layer_popup *
-create_popup(struct wlr_xdg_popup *wlr_popup, struct wlr_scene_tree *parent)
+create_popup(struct server *server, struct wlr_xdg_popup *wlr_popup,
+		struct wlr_scene_tree *parent)
 {
 	struct lab_layer_popup *popup = znew(*popup);
+	popup->server = server;
 	popup->wlr_popup = wlr_popup;
 	popup->scene_tree =
 		wlr_scene_xdg_surface_create(parent, wlr_popup->base);
@@ -433,7 +441,8 @@ popup_handle_new_popup(struct wl_listener *listener, void *data)
 	struct lab_layer_popup *lab_layer_popup =
 		wl_container_of(listener, lab_layer_popup, new_popup);
 	struct wlr_xdg_popup *wlr_popup = data;
-	struct lab_layer_popup *new_popup = create_popup(wlr_popup,
+	struct lab_layer_popup *new_popup = create_popup(
+		lab_layer_popup->server, wlr_popup,
 		lab_layer_popup->scene_tree);
 
 	if (!new_popup) {
@@ -500,7 +509,8 @@ handle_new_popup(struct wl_listener *listener, void *data)
 		.width = output_box.width,
 		.height = output_box.height,
 	};
-	struct lab_layer_popup *popup = create_popup(wlr_popup, surface->tree);
+	struct lab_layer_popup *popup =
+		create_popup(server, wlr_popup, surface->tree);
 	if (!popup) {
 		wl_resource_post_no_memory(wlr_popup->resource);
 		wlr_xdg_popup_destroy(wlr_popup);
