@@ -28,15 +28,16 @@
 #include <wlr/xwayland.h>
 #include "xwayland-shell-v1-protocol.h"
 #endif
+
 #include "drm-lease-v1-protocol.h"
 #include "common/macros.h"
+#include "common/scaled-scene-buffer.h"
 #include "config/rcxml.h"
 #include "config/session.h"
 #include "decorations.h"
-#if HAVE_LIBSFDO
 #include "desktop-entry.h"
-#endif
 #include "idle.h"
+#include "input/keyboard.h"
 #include "labwc.h"
 #include "layers.h"
 #include "magnifier.h"
@@ -64,6 +65,7 @@ static struct wl_event_source *sigchld_source;
 static void
 reload_config_and_theme(struct server *server)
 {
+	scaled_scene_buffer_invalidate_sharing();
 	rcxml_finish();
 	rcxml_read(rc.config_file);
 	theme_finish(server->theme);
@@ -92,6 +94,7 @@ handle_sighup(int signal, void *data)
 {
 	struct server *server = data;
 
+	keyboard_cancel_all_keybind_repeats(&server->seat);
 	session_environment_init();
 	reload_config_and_theme(server);
 	output_virtual_update_fallback(server);
@@ -556,10 +559,11 @@ server_init(struct server *server)
 	 * | Type              | Scene Tree       | Per Output | Example
 	 * | ----------------- | ---------------- | ---------- | -------
 	 * | ext-session       | lock-screen      | Yes        | swaylock
+	 * | osd               | osd_tree         | Yes        |
+	 * | compositor-menu   | menu_tree        | No         | root-menu
 	 * | layer-shell       | layer-popups     | Yes        |
 	 * | layer-shell       | overlay-layer    | Yes        |
 	 * | layer-shell       | top-layer        | Yes        | waybar
-	 * | server            | labwc-menus      | No         |
 	 * | xwayland-OR       | unmanaged        | No         | dmenu
 	 * | xdg-popups        | xdg-popups       | No         |
 	 * | toplevels windows | always-on-top    | No         |
@@ -748,9 +752,9 @@ server_finish(struct server *server)
 		wl_event_source_remove(sighup_source);
 	}
 	wl_display_destroy_clients(server->wl_display);
+	wlr_backend_destroy(server->backend);
 	wlr_allocator_destroy(server->allocator);
 	wlr_renderer_destroy(server->renderer);
-	wlr_backend_destroy(server->backend);
 	seat_finish(server);
 	workspaces_destroy(server);
 	wlr_scene_node_destroy(&server->scene->tree.node);
