@@ -7,11 +7,12 @@
 #include "config.h"
 #include "common/mem.h"
 #include "common/scaled-font-buffer.h"
+#include "common/scaled-icon-buffer.h"
+#include "common/scaled-img-buffer.h"
 #include "common/scene-helpers.h"
 #include "common/string-helpers.h"
-#if HAVE_LIBSFDO
 #include "desktop-entry.h"
-#endif
+#include "img/img.h"
 #include "labwc.h"
 #include "node.h"
 #include "ssd-internal.h"
@@ -73,20 +74,20 @@ ssd_titlebar_create(struct ssd *ssd)
 		int y = (theme->titlebar_height - theme->window_button_height) / 2;
 
 		wl_list_for_each(b, &rc.title_buttons_left, link) {
-			struct lab_data_buffer **buffers =
-				theme->window[active].buttons[b->type];
+			struct lab_img **imgs =
+				theme->window[active].button_imgs[b->type];
 			add_scene_button(&subtree->parts, b->type, parent,
-				buffers, x, y, view);
+				imgs, x, y, view);
 			x += theme->window_button_width + theme->window_button_spacing;
 		}
 
 		x = width - theme->window_titlebar_padding_width + theme->window_button_spacing;
 		wl_list_for_each_reverse(b, &rc.title_buttons_right, link) {
 			x -= theme->window_button_width + theme->window_button_spacing;
-			struct lab_data_buffer **buffers =
-				theme->window[active].buttons[b->type];
+			struct lab_img **imgs =
+				theme->window[active].button_imgs[b->type];
 			add_scene_button(&subtree->parts, b->type, parent,
-				buffers, x, y, view);
+				imgs, x, y, view);
 		}
 	} FOR_EACH_END
 
@@ -125,12 +126,14 @@ update_button_state(struct ssd_button *button, enum lab_button_state state,
 		button->state_set &= ~state;
 	}
 	/* Switch the displayed icon buffer to the new one */
-	for (uint8_t state_set = 0; state_set <= LAB_BS_ALL; state_set++) {
-		if (!button->nodes[state_set]) {
+	for (uint8_t state_set = LAB_BS_DEFAULT;
+			state_set <= LAB_BS_ALL; state_set++) {
+		struct scaled_img_buffer *buffer = button->img_buffers[state_set];
+		if (!buffer) {
 			continue;
 		}
-		wlr_scene_node_set_enabled(
-			button->nodes[state_set], button->state_set == state_set);
+		wlr_scene_node_set_enabled(&buffer->scene_buffer->node,
+			state_set == button->state_set);
 	}
 }
 
@@ -476,7 +479,7 @@ ssd_update_title(struct ssd *ssd)
 		dstate = active ? &state->active : &state->inactive;
 		text_color = theme->window[active].label_text_color;
 		bg_color = theme->window[active].title_bg_color;
-		font = &rc.font_activewindow;
+		font = active ?  &rc.font_activewindow : &rc.font_inactivewindow;
 
 		if (title_bg_width <= 0) {
 			dstate->truncated = true;
@@ -504,7 +507,7 @@ ssd_update_title(struct ssd *ssd)
 		if (part->buffer) {
 			scaled_font_buffer_update(part->buffer, title,
 				title_bg_width, font,
-				text_color, bg_color, NULL);
+				text_color, bg_color);
 		}
 
 		/* And finally update the cache */
@@ -583,37 +586,6 @@ ssd_update_window_icon(struct ssd *ssd)
 	free(ssd->state.app_id);
 	ssd->state.app_id = xstrdup(app_id);
 
-	struct theme *theme = ssd->view->server->theme;
-
-	/*
-	 * Ensure a small amount of horizontal padding within the button
-	 * area (2px on each side with the default 26px button width).
-	 * A new theme setting could be added to configure this. Using
-	 * an existing setting (padding.width or window.button.spacing)
-	 * was considered, but these settings have distinct purposes
-	 * already and are zero by default.
-	 */
-	int hpad = theme->window_button_width / 10;
-	int icon_size = MIN(theme->window_button_width - 2 * hpad,
-		theme->window_button_height);
-
-	/*
-	 * Load/render icons at the max scale of any usable output (at
-	 * this point in time). We don't want to be constantly reloading
-	 * icons as views are moved between outputs.
-	 *
-	 * TODO: currently there's no signal to reload/render icons if
-	 * outputs are reconfigured and the max scale changes.
-	 */
-	float icon_scale = output_max_scale(ssd->view->server);
-
-	struct lab_data_buffer *icon_buffer = desktop_entry_icon_lookup(
-		ssd->view->server, app_id, icon_size, icon_scale);
-	if (!icon_buffer) {
-		wlr_log(WLR_DEBUG, "icon could not be loaded for %s", app_id);
-		return;
-	}
-
 	struct ssd_sub_tree *subtree;
 	FOR_EACH_STATE(ssd, subtree) {
 		struct ssd_part *part = ssd_get_part(
@@ -622,17 +594,10 @@ ssd_update_window_icon(struct ssd *ssd)
 			break;
 		}
 
-		/* Replace all the buffers in the button with the window icon */
 		struct ssd_button *button = node_ssd_button_from_node(part->node);
-		for (uint8_t state_set = 0; state_set <= LAB_BS_ALL; state_set++) {
-			if (button->nodes[state_set]) {
-				update_window_icon_buffer(button->nodes[state_set],
-					icon_buffer);
-			}
-		}
+		assert(button->window_icon);
+		scaled_icon_buffer_set_app_id(button->window_icon, app_id);
 	} FOR_EACH_END
-
-	wlr_buffer_drop(&icon_buffer->base);
 #endif
 }
 
