@@ -331,6 +331,7 @@ handle_destroy(struct wl_listener *listener, void *data)
 
 	/* Remove xdg-shell view specific listeners */
 	wl_list_remove(&xdg_toplevel_view->set_app_id.link);
+	wl_list_remove(&xdg_toplevel_view->request_show_window_menu.link);
 	wl_list_remove(&xdg_toplevel_view->new_popup.link);
 	wl_list_remove(&view->commit.link);
 
@@ -660,10 +661,10 @@ xdg_toplevel_view_get_string_prop(struct view *view, const char *prop)
 	}
 
 	if (!strcmp(prop, "title")) {
-		return xdg_toplevel->title;
+		return xdg_toplevel->title ? xdg_toplevel->title : "";
 	}
 	if (!strcmp(prop, "app_id")) {
-		return xdg_toplevel->app_id;
+		return xdg_toplevel->app_id ? xdg_toplevel->app_id : "";
 	}
 	return "";
 }
@@ -874,7 +875,7 @@ xdg_activation_handle_request(struct wl_listener *listener, void *data)
 		return;
 	}
 
-	if (view->server->osd_state.cycle_view) {
+	if (view->server->input_mode == LAB_INPUT_STATE_WINDOW_SWITCHER) {
 		wlr_log(WLR_INFO, "Preventing focus request while in window switcher");
 		return;
 	}
@@ -933,7 +934,7 @@ xdg_toplevel_new(struct wl_listener *listener, void *data)
 		free(xdg_toplevel_view);
 		return;
 	}
-	view->scene_node = &tree->node;
+	view->content_tree = tree;
 	node_descriptor_create(&view->scene_tree->node,
 		LAB_NODE_DESC_VIEW, view);
 
